@@ -1,11 +1,11 @@
-;;;; Copyright: (c) copyright 1996 by Gilbert Baumann, distributed under GPL.
+;;;; Copyright (C) 1996 by Gilbert Baumann
+;;;; Copyright (C) 2001-2007 by Sam Steingold
+;;;; Distributed under GPL.
 ;;;; Some parts are from the MIT-CLX Distribution, copyrighted by
 ;;;; Texas Instruments Incorporated, but freely distributable
 ;;;; for details see image.lisp or the MIT-CLX distribution.
 
-(defpackage "XLIB"
-  ;; (:use "COMMON-LISP" "CLOS")
-  (:import-from "SYS" "STRING-CONCAT"))
+(defpackage #:xlib)
 
 (provide "clx")
 
@@ -48,6 +48,7 @@
    define-keysym-set delete-property delete-resource destroy-subwindows
    destroy-window device-busy device-event-mask device-event-mask-class
    discard-current-event discard-font-info display display-after-function
+   display-authorization        ; extension
    display-authorization-data display-authorization-name
    display-bitmap-format display-byte-order display-default-screen
    display-display display-error-handler display-finish-output
@@ -124,8 +125,8 @@
    screen-min-installed-maps screen-p screen-plist screen-root
    screen-root-depth screen-root-visual screen-root-visual-info
    screen-save-unders-p screen-saver screen-white-pixel screen-width
-   screen-width-in-millimeters seg-seq selection-owner send-event
-   sequence-error set-input-focus
+   screen-width-in-millimeters seg-seq selection-owner set-selection-owner
+   send-event sequence-error set-input-focus
    set-modifier-mapping  set-screen-saver
    set-standard-colormap set-standard-properties
    set-wm-class set-wm-properties set-wm-resources state-keysym-p
@@ -168,6 +169,11 @@
    untrace-display show-trace
    display-trace ; for backwards compatibility describe-request describe-event describe-reply
    closed-display-p
+   ;; extensions
+   open-default-display
+   ;; display-xdefaults resource-database-to-string resource-database-of-string
+   ;; resource-database-locale
+   ;; not implemented
    describe-error describe-trace))
 
 ;;; SHAPE extension
@@ -178,7 +184,7 @@
 ;;;;  Types
 ;;;; --------------------------------------------------------------------------
 ;;;;
-;;;; Lots of deftypes randomly gathers from MIT-CLX implementation
+;;;; Lots of deftypes randomly gathered from MIT-CLX implementation
 ;;;;
 
 (deftype card4 ()       '(unsigned-byte 4))    ;not exported
@@ -323,8 +329,9 @@
   red-mask green-mask blue-mask
   bits-per-rgb
   colormap-entries
-  ;; There appears also a plist and a display slot in the MIT-CLX, but not in the manual
-  ;; To what should we be compatible?!
+  ;; There appears also a plist and a display slot in the MIT-CLX,
+  ;; but not in the manual!
+  ;; With what should we be compatible?!
   ;; plist display
   )
 
@@ -333,15 +340,34 @@
                     (:copier nil)
                     (:conc-name %display-))
   foreign-pointer ;; these two slots are for use in clx.d only.
-  hash-table      ;; .. so leave hands off here!
+  hash-table      ;; .. so keep hands off here!
   plist
   after-function
-  error-handler)
+  error-handler
+  display)
+
+;; (defstruct (resource-database (:constructor %make-rdb (foreign-pointer)))
+;;   (foreign-pointer nil :read-only t))
+;; (defstruct (search-table (:constructor %make-search-table (foreign-pointer)))
+;;   (foreign-pointer nil :read-only t))
 
 ;; ***************************************************************************
-;; ... CAUTION ending here.
+;; ... CAUTION ending here (resume careless coding)
 ;; ***************************************************************************
 
+;; (eval-when (:compile-toplevel)
+;;   (defpackage #:posix (:export #:mkstemp)))
+;; (defun resource-database-to-string (rdb)
+;;   (let ((tmp-file (posix:mkstemp "/tmp/rdb")))
+;;     (close tmp-file)
+;;     (write-resources rdb tmp-file)
+;;     (unwind-protect
+;;         (with-open-file (in tmp-file)
+;;           (let ((st (make-string (file-length in))))
+;;             (read-sequence st in)
+;;             st))
+;;       (delete-file tmp-file))))
+
 (defun make-color (&key (red 1.0s0) (green 1.0s0) (blue 1.0s0)
                    &allow-other-keys)
   (make-color-internal red green blue))
@@ -379,6 +405,8 @@
 (defsetf DISPLAY-AFTER-FUNCTION       SET-DISPLAY-AFTER-FUNCTION)
 (defsetf DISPLAY-ERROR-HANDLER        SET-DISPLAY-ERROR-HANDLER)
 (defsetf DISPLAY-PLIST                SET-DISPLAY-PLIST)
+(defsetf DISPLAY-DEFAULT-SCREEN       SET-DISPLAY-DEFAULT-SCREEN)
+(defsetf DISPLAY-XDEFAULTS            SET-DISPLAY-XDEFAULTS)
 (defsetf DRAWABLE-BORDER-WIDTH        SET-DRAWABLE-BORDER-WIDTH)
 (defsetf DRAWABLE-HEIGHT              SET-DRAWABLE-HEIGHT)
 (defsetf DRAWABLE-PLIST               SET-DRAWABLE-PLIST)
@@ -418,7 +446,7 @@
 (defsetf POINTER-MAPPING              SET-POINTER-MAPPING)
 (defsetf SCREEN-PLIST                 SET-SCREEN-PLIST)
 (defsetf SELECTION-OWNER (display selection &optional time) (owner)
-  `(SET-SELECTION-OWNER ,owner ,display ,selection ,time))
+  `(SET-SELECTION-OWNER ,display ,selection ,owner ,time))
 (defsetf WINDOW-BACKGROUND            SET-WINDOW-BACKGROUND)
 (defsetf WINDOW-BACKING-PIXEL         SET-WINDOW-BACKING-PIXEL)
 (defsetf WINDOW-BACKING-PLANES        SET-WINDOW-BACKING-PLANES)
@@ -746,7 +774,7 @@
 
 ;; XXX
 
-;; This code is buggy. My interpretation of chnage-property and get-property is
+;; This code is buggy. My interpretation of change-property and get-property is
 ;; that they only deal with unsigned data, but the as obsolete marked fields x
 ;; and y are signed, and the code below does not take care.  Running it
 ;; interpreted, hence with type checks gets errors.
@@ -1184,7 +1212,7 @@
 (defun cut-buffer (display &key (buffer 0) (type :STRING) (result-type 'string)
                    (transform #'card8->char) (start 0) end)
   ;; Return the contents of cut-buffer BUFFER
-  (let* ((root (screen-root (first (display-roots display))))
+  (let* ((root (screen-root (display-default-screen display)))
          (property (aref '#(:CUT_BUFFER0 :CUT_BUFFER1 :CUT_BUFFER2 :CUT_BUFFER3
                             :CUT_BUFFER4 :CUT_BUFFER5 :CUT_BUFFER6 :CUT_BUFFER7)
                          buffer)))
@@ -1192,20 +1220,23 @@
                   :start start :end end :transform transform)))
 
 (defun (setf cut-buffer) (data display &key (buffer 0) (type :STRING) (format 8)
-                               (start 0) end (transform #'char->card8))
-  (let* ((root (screen-root (first (display-roots display))))
+                          (start 0) end (transform #'char->card8))
+  (let* ((root (screen-root (display-default-screen display)))
          (property (aref '#(:CUT_BUFFER0 :CUT_BUFFER1 :CUT_BUFFER2 :CUT_BUFFER3
-                                         :CUT_BUFFER4 :CUT_BUFFER5 :CUT_BUFFER6 :CUT_BUFFER7)
+                            :CUT_BUFFER4 :CUT_BUFFER5 :CUT_BUFFER6 :CUT_BUFFER7)
                          buffer)))
-    (change-property root property data type format :transform transform :start start :end end)
+    (change-property root property data type format
+                     :start start :end end :transform transform)
     data))
 
 (defun rotate-cut-buffers (display &optional (delta 1) (careful-p t))
-  ;; Positive rotates left, negative rotates right (opposite of actual protocol request).
-  ;; When careful-p, ensure all cut-buffer properties are defined, to prevent errors.
-  (let* ((root (screen-root (first (display-roots display))))
-         (buffers '#(:cut_buffer0 :cut_buffer1 :cut_buffer2 :cut_buffer3
-                     :cut_buffer4 :cut_buffer5 :cut_buffer6 :cut_buffer7)))
+  ;; Positive rotates left, negative rotates right
+  ;; (opposite of actual protocol request).
+  ;; When careful-p, ensure all cut-buffer properties are defined,
+  ;; to prevent errors.
+  (let* ((root (screen-root (display-default-screen display)))
+         (buffers '#(:CUT_BUFFER0 :CUT_BUFFER1 :CUT_BUFFER2 :CUT_BUFFER3
+                     :CUT_BUFFER4 :CUT_BUFFER5 :CUT_BUFFER6 :CUT_BUFFER7)))
     (when careful-p
       (let ((props (list-properties root)))
         (dotimes (i 8)
@@ -1219,8 +1250,10 @@
 ;;;; --------------------------------------------------------------------------
 
 ;;; NOTE:
-;;;   I used here a (funcall #,#'fun ..) klugde, but by clisp-1996-07-22 this now considered
-;;;   illegal, so I save the untraced functions by copying them. This allows me to trace all or arbitrary
+;;;   I used here a (funcall #,#'fun ..) klugde,
+;;;   but by clisp-1996-07-22 this now considered illegal,
+;;;   so I save the untraced functions by copying them.
+;;;   This allows me to trace all or arbitrary
 ;;;   xlib functions without getting into infinite recursion.
 
 (setf (fdefinition '%untraced-color-blue) #'color-blue
@@ -1304,6 +1337,59 @@
 ;;;;  Misc
 ;;;; --------------------------------------------------------------------------
 
+;;; from dependent.lisp in http://common-lisp.net/~crhodes/clx
+;;; this particular defaulting behaviour is typical to most Unices, I think
+(defun get-default-display (&optional display-name)
+  "Parse the argument DISPLAY-NAME, or the environment variable $DISPLAY
+if it is NIL.  Display names have the format
+
+  [protocol/] [hostname] : [:] displaynumber [.screennumber]
+
+There are two special cases in parsing, to match that done in the Xlib
+C language bindings
+
+ - If the hostname is ``unix'' or the empty string, any supplied
+   protocol is ignored and a connection is made using the :local
+   transport.
+
+ - If a double colon separates hostname from displaynumber, the
+   protocol is assumed to be decnet.
+
+Returns a list of (host display-number screen protocol)."
+  (let* ((name (or display-name
+                   (ext:getenv "DISPLAY")
+                   (error "DISPLAY environment variable is not set")))
+         (slash-i (or (position #\/ name) -1))
+         (colon-i (position #\: name :start (1+ slash-i)))
+         (decnet-colon-p (eql (elt name (1+ colon-i)) #\:))
+         (host (subseq name (1+ slash-i) colon-i))
+         (dot-i (and colon-i (position #\. name :start colon-i)))
+         (display (when colon-i
+                    (parse-integer name
+                                   :start (if decnet-colon-p
+                                              (+ colon-i 2)
+                                              (1+ colon-i))
+                                   :end dot-i)))
+         (screen (when dot-i
+                   (parse-integer name :start (1+ dot-i))))
+         (protocol
+          (cond ((or (string= host "") (string-equal host "unix")) :local)
+                (decnet-colon-p :decnet)
+                ((> slash-i -1) (intern
+                                 (string-upcase (subseq name 0 slash-i))
+                                 :keyword))
+                (t :internet))))
+    (list host (or display 0) (or screen 0) protocol)))
+
+(defun open-default-display (&optional display-name)
+  "Open a connection to DISPLAY-NAME if supplied, or to the appropriate
+default display as given by GET-DEFAULT-DISPLAY otherwise."
+  (destructuring-bind (host display screen protocol)
+      (get-default-display display-name)
+    (let ((dpy (open-display host :display display :protocol protocol)))
+       (setf (display-default-screen dpy) screen)
+       dpy)))
+
 
 ;;;; --------------------------------------------------------------------------
 ;;;;  Stuff, which is realy some internals of CLX,
@@ -1483,16 +1569,24 @@
 
 ;;; Error handler, we probably want a proper condition hierarchy, but for a first approach this may be enough:
 
-(defun default-error-handler (display error-code &key current-sequence sequence
-                              major minor resource-id atom-id value)
-  (cerror "Ignore this error and proceed."
-          "Asynchronous ~A on display ~S, sequence ~D. Opcode is ~D.~D. Current sequence is ~D.~A"
-          error-code display sequence major minor current-sequence
-          ;; at most one of RESOURCE-ID, ATOM-ID, and VALUE is available
-          (cond (resource-id (format nil " Resource ID is #x~8,'0x." resource-id))
-                (value (format nil " Bad value is ~D." value))
-                (atom-id (format nil " Bad atom ID is #x~8,'0x." atom-id))
-                (t ""))))
+(defun default-error-handler (display error-key &rest key-vals
+			      &key asynchronous &allow-other-keys)
+  (if asynchronous
+      (apply 'cerror "Ignore this error and proceed." error-key
+	     :display display :error-key error-key key-vals)
+      (apply 'error error-key
+	     :display display :error-key error-key key-vals)))
+
+(defun report-request-error (condition stream)
+  (let ((error-key (request-error-error-key condition))
+	(asynchronous (request-error-asynchronous condition))
+	(major (request-error-major condition))
+	(minor (request-error-minor condition))
+	(sequence (request-error-sequence condition))
+	(current-sequence (request-error-current-sequence condition)))
+    (format stream "~:[~;Asynchronous ~]~a in ~:[request ~d (last request was ~d) ~;current request~2* ~] Code ~d.~d"
+	    asynchronous error-key (= sequence current-sequence)
+	    sequence current-sequence major minor)))
 
 (define-condition x-error (error)
   ((caller :reader x-error-caller :initarg :caller)))
@@ -1505,34 +1599,226 @@
              (x-error-caller condition)
              (closed-display-display condition)))))
 
-(define-condition request-error (x-error) ())
-(define-condition resource-error (request-error) ())
+(define-condition request-error (x-error)
+  ((display :reader request-error-display :initarg :display)
+   (error-key :reader request-error-error-key :initarg :error-key)
+   (major :reader request-error-major :initarg :major)
+   (minor :reader request-error-minor :initarg :minor)
+   (sequence :reader request-error-sequence :initarg :sequence)
+   (current-sequence :reader request-error-current-sequence :initarg :current-sequence)
+   (asynchronous :reader request-error-asynchronous :initarg :asynchronous))
+  (:report report-request-error))
+
+(define-condition resource-error (request-error)
+  ((resource-id :reader resource-error-resource-id :initarg :resource-id))
+  (:report
+   (lambda (condition stream)
+     (report-request-error condition stream)
+     (format stream " ID #x~x" (resource-error-resource-id condition)))))
+
+(define-condition unknown-error (request-error)
+  ((error-code :reader unknown-error-error-code :initarg :error-code))
+  (:report
+   (lambda (condition stream)
+     (report-request-error condition stream)
+     (format stream " Error Code ~d." (unknown-error-error-code condition)))))
+
 (define-condition access-error (request-error) ())
+
 (define-condition alloc-error (request-error) ())
-(define-condition atom-error (request-error) ())
+
+(define-condition atom-error (request-error)
+  ((atom-id :reader atom-error-atom-id :initarg :atom-id))
+  (:report
+   (lambda (condition stream)
+     (report-request-error condition stream)
+     (format stream " Atom-ID #x~x" (atom-error-atom-id condition)))))
+
 (define-condition colormap-error (resource-error) ())
-(define-condition connection-failure (x-error) ())
+
 (define-condition cursor-error (resource-error) ())
-(define-condition device-busy (x-error) ())
+
 (define-condition drawable-error (resource-error) ())
+
 (define-condition font-error (resource-error) ())
+
 (define-condition gcontext-error (resource-error) ())
+
 (define-condition id-choice-error (resource-error) ())
-(define-condition implementation-error (resource-error) ())
-(define-condition length-error (resource-error) ())
-(define-condition lookup-error (x-error) ())
+
+(define-condition illegal-request-error (request-error) ())
+
+(define-condition length-error (request-error) ())
+
 (define-condition match-error (request-error) ())
-(define-condition missing-parameter (x-error) ())
+
 (define-condition name-error (request-error) ())
+
 (define-condition pixmap-error (resource-error) ())
-(define-condition reply-length-error (x-error) ())
-(define-condition reply-timeout (x-error) ())
-(define-condition sequence-error (x-error) ())
-(define-condition server-disconnect (x-error) ())
-(define-condition unexpected-reply (x-error) ())
-(define-condition unknown-error (request-error) ())
-(define-condition value-error (request-error) ())
-(define-condition window-error (resource-error) ())
+
+(define-condition value-error (request-error)
+  ((value :reader value-error-value :initarg :value))
+  (:report
+    (lambda (condition stream)
+      (report-request-error condition stream)
+      (format stream " Value ~d." (value-error-value condition)))))
+
+(define-condition x-type-error (type-error x-error)
+  ((type-string :reader x-type-error-type-string :initarg :type-string))
+  (:report
+    (lambda (condition stream)
+      (format stream "~s isn't a ~a"
+	      (type-error-datum condition)
+	      (or (x-type-error-type-string condition)
+		  (type-error-expected-type condition))))))
+
+(define-condition closed-display (x-error)
+  ((display :reader closed-display-display :initarg :display))
+  (:report
+    (lambda (condition stream)
+      (format stream "Attempt to use closed display ~s"
+	      (closed-display-display condition)))))
+
+(define-condition lookup-error (x-error)
+  ((id :reader lookup-error-id :initarg :id)
+   (display :reader lookup-error-display :initarg :display)
+   (type :reader lookup-error-type :initarg :type)
+   (object :reader lookup-error-object :initarg :object))
+  (:report
+    (lambda (condition stream)
+      (format stream "ID ~d from display ~s should have been a ~s, but was ~s"
+	      (lookup-error-id condition)
+	      (lookup-error-display condition)
+	      (lookup-error-type condition)
+	      (lookup-error-object condition)))))
+
+(define-condition connection-failure (x-error)
+  ((major-version :reader connection-failure-major-version :initarg :major-version)
+   (minor-version :reader connection-failure-minor-version :initarg :minor-version)
+   (host :reader connection-failure-host :initarg :host)
+   (display :reader connection-failure-display :initarg :display)
+   (reason :reader connection-failure-reason :initarg :reason))
+  (:report
+    (lambda (condition stream)
+      (format stream "Connection failure to X~d.~d server ~a display ~d: ~a"
+	      (connection-failure-major-version condition)
+	      (connection-failure-minor-version condition)
+	      (connection-failure-host condition)
+	      (connection-failure-display condition)
+	      (connection-failure-reason condition)))))
+
+(define-condition reply-length-error (x-error)
+  ((reply-length :reader reply-length-error-reply-length :initarg :reply-length)
+   (expected-length :reader reply-length-error-expected-length :initarg :expected-length)
+   (display :reader reply-length-error-display :initarg :display))
+  (:report
+    (lambda (condition stream)
+      (format stream "Reply length was ~d when ~d words were expected for display ~s"
+	      (reply-length-error-reply-length condition)
+	      (reply-length-error-expected-length condition)
+	      (reply-length-error-display condition)))))
+
+(define-condition reply-timeout (x-error)
+  ((timeout :reader reply-timeout-timeout :initarg :timeout)
+   (display :reader reply-timeout-display :initarg :display))
+  (:report
+    (lambda (condition stream)
+      (format stream "Timeout after waiting ~d seconds for a reply for display ~s"
+	      (reply-timeout-timeout condition)
+	      (reply-timeout-display condition)))))
+
+(define-condition sequence-error (x-error)
+  ((display :reader sequence-error-display :initarg :display)
+   (req-sequence :reader sequence-error-req-sequence :initarg :req-sequence)
+   (msg-sequence :reader sequence-error-msg-sequence :initarg :msg-sequence))
+  (:report
+    (lambda (condition stream)
+      (format stream "Reply out of sequence for display ~s.~%  Expected ~d, Got ~d"
+	      (sequence-error-display condition)
+	      (sequence-error-req-sequence condition)
+	      (sequence-error-msg-sequence condition)))))
+
+(define-condition unexpected-reply (x-error)
+  ((display :reader unexpected-reply-display :initarg :display)
+   (msg-sequence :reader unexpected-reply-msg-sequence :initarg :msg-sequence)
+   (req-sequence :reader unexpected-reply-req-sequence :initarg :req-sequence)
+   (length :reader unexpected-reply-length :initarg :length))
+  (:report
+    (lambda (condition stream)
+      (format stream "Display ~s received a server reply when none was expected.~@
+		      Last request sequence ~d Reply Sequence ~d Reply Length ~d bytes."
+	      (unexpected-reply-display condition)
+	      (unexpected-reply-req-sequence condition)
+	      (unexpected-reply-msg-sequence condition)
+	      (unexpected-reply-length condition)))))
+
+(define-condition missing-parameter (x-error)
+  ((parameter :reader missing-parameter-parameter :initarg :parameter))
+  (:report
+    (lambda (condition stream)
+      (let ((parm (missing-parameter-parameter condition)))
+	(if (consp parm)
+	    (format stream "One or more of the required parameters ~a is missing."
+		    parm)
+	  (format stream "Required parameter ~a is missing or null." parm))))))
+
+;; This can be signalled anywhere a pseudo font access fails.
+(define-condition invalid-font (x-error)
+  ((font :reader invalid-font-font :initarg :font))
+  (:report
+    (lambda (condition stream)
+      (format stream "Can't access font ~s" (invalid-font-font condition)))))
+
+(define-condition device-busy (x-error)
+  ((display :reader device-busy-display :initarg :display))
+  (:report
+    (lambda (condition stream)
+      (format stream "Device busy for display ~s"
+	      (device-busy-display condition)))))
+
+(define-condition unimplemented-event (x-error)
+  ((display :reader unimplemented-event-display :initarg :display)
+   (event-code :reader unimplemented-event-event-code :initarg :event-code))
+  (:report
+    (lambda (condition stream)
+      (format stream "Event code ~d not implemented for display ~s"
+	      (unimplemented-event-event-code condition)
+	      (unimplemented-event-display condition)))))
+
+(define-condition undefined-event (x-error)
+  ((display :reader undefined-event-display :initarg :display)
+   (event-name :reader undefined-event-event-name :initarg :event-name))
+  (:report
+    (lambda (condition stream)
+      (format stream "Event code ~d undefined for display ~s"
+	      (undefined-event-event-name condition)
+	      (undefined-event-display condition)))))
+
+(define-condition absent-extension (x-error)
+  ((name :reader absent-extension-name :initarg :name)
+   (display :reader absent-extension-display :initarg :display))
+  (:report
+    (lambda (condition stream)
+      (format stream "Extension ~a isn't defined for display ~s"
+	      (absent-extension-name condition)
+	      (absent-extension-display condition)))))
+
+(define-condition inconsistent-parameters (x-error)
+  ((parameters :reader inconsistent-parameters-parameters :initarg :parameters))
+  (:report
+    (lambda (condition stream)
+      (format stream "inconsistent-parameters:~{ ~s~}"
+	      (inconsistent-parameters-parameters condition)))))
+
+(define-condition window-error (resource-error)())
+
+(define-condition implementation-error (request-error) ())
+
+(define-condition connection-failure (x-error) ())
+
+(define-condition device-busy (x-error) ())
+
+;; (define-condition server-disconnect (x-error) ())
 
 (pushnew "XLIB" custom:*system-package-list* :test #'string=)
 (pushnew "XPM" custom:*system-package-list* :test #'string=)
@@ -1543,13 +1829,9 @@
                 (declare (ignore args))
                 (progn (warn "~S is not implemented: ~S" ',name form)
                        form))))
-(undefined DISPLAY-RESOURCE-ID-BASE)
-(undefined DISPLAY-RESOURCE-ID-MASK)
 (undefined DISPLAY-TRACE)
 (undefined DRAW-GLYPH)
 (undefined DRAW-IMAGE-GLYPH)
 (undefined TRANSLATE-DEFAULT)
 (undefined QUEUE-EVENT)
-(undefined CHANGE-KEYBOARD-MAPPING)
-(undefined KEYBOARD-MAPPING)
 )
