@@ -1,6 +1,6 @@
 ;; CLISP interface to PARI <http://pari.math.u-bordeaux.fr/>
 ;; Copyright (C) 1995 Michael Stoll
-;; Copyright (C) 2004-2005 Sam Steingold
+;; Copyright (C) 2004-2008 Sam Steingold
 ;; This is free software, distributed under the GNU GPL
 
 (defpackage "PARI"
@@ -9,57 +9,12 @@
 (in-package "PARI")
 (pushnew "PARI" custom:*system-package-list* :test #'string=)
 
+(setf (documentation (find-package "PARI") 'sys::impnotes) "pari")
+
 (default-foreign-language :stdc)
 
 (pushnew :pari *features*)
-
-;;; Define the functions for output and error output from pari
-
-(defun pari-putc (c) (write-char c *standard-output*))
-(defun pari-puts (s) (write-string s *standard-output*))
-(defun pari-flush () (finish-output *standard-output*))
-
-(def-call-in pari-putc
-  (:name "clispPutc")
-  (:return-type nil)
-  (:arguments (c character))
-  (:language :stdc))
-(def-call-in pari-puts
-  (:name "clispPuts")
-  (:return-type nil)
-  (:arguments (s c-string :in :malloc-free))
-  (:language :stdc))
-(def-call-in pari-flush
-  (:name "clispFlush")
-  (:return-type nil)
-  (:arguments)
-  (:language :stdc))
-
-(defun pari-err-putc (c) (write-char c *error-output*))
-(defun pari-err-puts (s) (write-string s *error-output*))
-(defun pari-err-flush () (finish-output *error-output*))
-(defun pari-err-die () (error "Error within pari."))
-
-(def-call-in pari-err-putc
-  (:name "clispErrPutc")
-  (:return-type nil)
-  (:arguments (c character))
-  (:language :stdc))
-(def-call-in pari-err-puts
-  (:name "clispErrPuts")
-  (:return-type nil)
-  (:arguments (s c-string :in :malloc-free))
-  (:language :stdc))
-(def-call-in pari-err-flush
-  (:name "clispErrFlush")
-  (:return-type nil)
-  (:arguments)
-  (:language :stdc))
-(def-call-in pari-err-die
-  (:name "clispErrDie")
-  (:return-type nil)
-  (:arguments)
-  (:language :stdc))
+(provide "pari")
 
 ;;; Declare all the pari types, variables, functions, ...
 (c-lines "#undef T~%#include <pari/pari.h>~%")
@@ -183,7 +138,7 @@
 (def-c-var pari-i    (:name "gi")    (:type pari-gen) (:read-only t))
 (def-c-var pari-0    (:name "gen_0") (:type pari-gen) (:read-only t))
 (def-c-var pari--1   (:name "gen_m1") (:type pari-gen) (:read-only t))
-(def-c-var pari-nil  (:name "gnil")  (:type pari-gen) (:read-only t))
+;;(def-c-var pari-nil  (:name "gnil")  (:type pari-gen) (:read-only t))
 
 ;; extern  GEN *polun,*polx;
 (def-c-var pari-poly-1 (:name "polun") (:type (c-ptr pari-gen)) (:read-only t))
@@ -234,22 +189,21 @@
          (format nil "~A corresponds to the gp function ~A:~%~A"
                  name str doc))))
 
-(c-lines "char *pariversion=PARIVERSION;~%int pari_version_code=PARI_VERSION_CODE;~%int pari_version_shift=PARI_VERSION_SHIFT;~%")
-(def-c-var pariversion (:type c-string) (:read-only t))
-(def-c-var pari_version_code (:type int) (:read-only t))
-(def-c-var pari_version_shift (:type int) (:read-only t))
+(def-c-const PARIVERSION (:type c-string))
+(def-c-const PARI_VERSION_CODE)
+(def-c-const PARI_VERSION_SHIFT)
 (defconstant pari-version
-  (let ((code pari_version_code) (shift pari_version_shift))
-    (list pariversion
-          (ldb (byte shift (* 2 shift)) code)
-          (ldb (byte shift shift) code)
-          (ldb (byte shift 0) code))))
+  (list PARIVERSION
+        (ldb (byte PARI_VERSION_SHIFT (* 2 PARI_VERSION_SHIFT))
+             PARI_VERSION_CODE)
+        (ldb (byte PARI_VERSION_SHIFT PARI_VERSION_SHIFT) PARI_VERSION_CODE)
+        (ldb (byte PARI_VERSION_SHIFT 0) PARI_VERSION_CODE)))
 (export 'pari-version)
 
 ;;; /* init.c */
 
 ;; long allocatemoremem(size_t newsize);
-(def-call-out allocatemoremem (:arguments (newsize ulong)) (:return-type long))
+(def-call-out allocatemoremem (:arguments (newsize size_t)) (:return-type long))
 (defun allocatemem (&optional (newsize 0)) (allocatemoremem newsize))
 
 (c-lines "#include \"cpari.h\"~%")
