@@ -1,7 +1,7 @@
 /*
  * PCRE - Perl Compatible Regular Expressions
  * <http://www.pcre.org/>
- * Copyright (C) 2003-2007 Sam Steingold
+ * Copyright (C) 2003-2008 Sam Steingold
  * GPL2
  */
 
@@ -80,9 +80,9 @@ DEFCHECKER(pcre_options,prefix=PCRE,bitmasks=both,\
            CASELESS MULTILINE DOTALL EXTENDED ANCHORED DOLLAR-ENDONLY EXTRA \
            NOTBOL NOTEOL UNGREEDY NOTEMPTY UTF8 NO-AUTO-CAPTURE NO-UTF8-CHECK \
            AUTO-CALLOUT PARTIAL)
-DEFUN(PCRE:PCRE-COMPILE,string &key :STUDY :IGNORE-CASE :MULTILINE :DOTALL \
-      :EXTENDED :ANCHORED :DOLLAR-ENDONLY :EXTRA :NOTBOL :NOTEOL :UNGREADY \
-      :NOTEMPTY :NO-AUTO-CAPTURE :AUTO-CALLOUT :PARTIAL)
+DEFUN(PCRE:PCRE-COMPILE,string &key STUDY IGNORE-CASE MULTILINE DOTALL \
+      EXTENDED ANCHORED DOLLAR-ENDONLY EXTRA NOTBOL NOTEOL UNGREADY \
+      NOTEMPTY NO-AUTO-CAPTURE AUTO-CALLOUT PARTIAL)
 { /* compile the pattern, return PATTERN struct */
   int options = PCRE_UTF8 | pcre_compile_flags();
   bool study = !missingp(STACK_0);
@@ -102,7 +102,7 @@ DEFUN(PCRE:PCRE-COMPILE,string &key :STUDY :IGNORE-CASE :MULTILINE :DOTALL \
     pushSTACK(safe_to_string(error_message));
     pushSTACK(fixnum(error_offset));
     pushSTACK(*string); pushSTACK(TheSubr(subr_self)->name);
-    check_value(error,GETTEXT("~S(~S) at ~S: ~S"));
+    check_value(error_condition,GETTEXT("~S(~S) at ~S: ~S"));
     *string = value1;
     goto pcre_compile_restart;
   }
@@ -117,7 +117,7 @@ DEFUN(PCRE:PCRE-COMPILE,string &key :STUDY :IGNORE-CASE :MULTILINE :DOTALL \
       STACK_0 = NIL;           /* no PLACE - discard compiled_pattern */
       pushSTACK(asciz_to_string(error_message,GLO(misc_encoding)));
       pushSTACK(*string); pushSTACK(TheSubr(subr_self)->name);
-      check_value(error,"~S(~S): ~S");
+      check_value(error_condition,"~S(~S): ~S");
       *string = value1;
       goto pcre_compile_restart;
     }
@@ -129,27 +129,27 @@ DEFUN(PCRE:PCRE-COMPILE,string &key :STUDY :IGNORE-CASE :MULTILINE :DOTALL \
 }
 
 /* can trigger GC */
-static void check_pattern (object pat, pcre** compiled_pattern,
+static void check_pattern (gcv_object_t *pat, pcre** compiled_pattern,
                            pcre_extra** study)
 { /* extract compiled pattern and the study results from the PATTERN */
-  pat = check_classname(pat,`PCRE::PATTERN`);
+  *pat = check_classname(*pat,`PCRE::PATTERN`);
   /* FIXME for derived structs! */
   *compiled_pattern =
-    (pcre*)TheFpointer(TheStructure(pat)->recdata[1])->fp_pointer;
-  if (nullp(TheStructure(pat)->recdata[2])) *study = NULL;
+    (pcre*)TheFpointer(TheStructure(*pat)->recdata[1])->fp_pointer;
+  if (nullp(TheStructure(*pat)->recdata[2])) *study = NULL;
   else *study =
-    (pcre_extra*)TheFpointer(TheStructure(pat)->recdata[2])->fp_pointer;
+    (pcre_extra*)TheFpointer(TheStructure(*pat)->recdata[2])->fp_pointer;
 }
 
 /* two objects should be on STACK for the error message */
 DEFCHECKER(error_pcre_code,prefix=PCRE_ERROR, NOMATCH NULL BADOPTION    \
            BADMAGIC UNKNOWN_NODE NOMEMORY NOSUBSTRING MATCHLIMIT CALLOUT \
            DFA_UITEM DFA_UCOND DFA_UMLIMIT DFA_WSSIZE DFA_RECURSE       \
-           BADUTF8 BADUTF8_OFFSET PARTIAL BADPARTIAL INTERNAL BADCOUNT)
+           BADUTF8 BADUTF8_OFFSET PARTIAL BADPARTIAL :INTERNAL BADCOUNT)
 nonreturning_function(static, error_pcre, (int status)) {
   pushSTACK(error_pcre_code_reverse(status));
   pushSTACK(sfixnum(status)); pushSTACK(TheSubr(subr_self)->name);
-  fehler(error,"~S/~S=~S: ~S ~S");
+  error(error_condition,"~S/~S=~S: ~S ~S");
 }
 
 
@@ -159,7 +159,7 @@ nonreturning_function(static, error_pcre, (int status)) {
     end_system_call();                                  \
     if (status < bad) error_pcre(status);               \
   } while(0)
-DEFCHECKER(fullinfo_arg,prefix=PCRE_INFO,OPTIONS SIZE CAPTURECOUNT      \
+DEFCHECKER(fullinfo_arg,prefix=PCRE_INFO,OPTIONS :SIZE CAPTURECOUNT     \
            BACKREFMAX FIRSTBYTE FIRSTTABLE LASTLITERAL                  \
            NAMEENTRYSIZE NAMECOUNT NAMETABLE STUDYSIZE)
 /* PCRE_INFO_DEFAULTTABLES -- does not look useful
@@ -238,11 +238,11 @@ DEFUN(PCRE:PATTERN-INFO,pattern &optional request)
 {
   pcre *c_pat;
   pcre_extra *study;
-  check_pattern(STACK_1,&c_pat,&study);
+  check_pattern(&STACK_1,&c_pat,&study);
   if (missingp(STACK_0)) {
     int count = 0;
     pushSTACK(`:OPTIONS`); pushSTACK(fullinfo_options(c_pat,study)); count+=2;
-    pushSTACK(`:SIZE`);
+    pushSTACK(S(Ksize));
     pushSTACK(fullinfo_size(c_pat,study,PCRE_INFO_SIZE)); count+=2;
     pushSTACK(`:CAPTURECOUNT`);
     pushSTACK(fullinfo_int(c_pat,study,PCRE_INFO_CAPTURECOUNT)); count+=2;
@@ -313,7 +313,7 @@ DEFUN(PCRE:PCRE-NAME-TO-INDEX,pattern name)
   pcre *c_pat;
   pcre_extra *study;
   int index;
-  check_pattern(STACK_1,&c_pat,&study);
+  check_pattern(&STACK_1,&c_pat,&study);
 #if defined(HAVE_PCRE_GET_STRINGNUMBER)
  restart_pcre_get_stringnumber:
   with_string_0(check_string(STACK_0),GLO(misc_encoding),name, {
@@ -324,21 +324,21 @@ DEFUN(PCRE:PCRE-NAME-TO-INDEX,pattern name)
     pushSTACK(NIL);             /* no PLACE */
     pushSTACK(STACK_1);         /* name */
     pushSTACK(TheSubr(subr_self)->name);
-    check_value(error,GETTEXT("~S: ~S is not a valid pattern name"));
+    check_value(error_condition,GETTEXT("~S: ~S is not a valid pattern name"));
     STACK_0 = value1;
     goto restart_pcre_get_stringnumber;
   }
   skipSTACK(2);
 #else /* PCRE versions before 4.0 did not have pcre_get_stringnumber() */
   pushSTACK(TheSubr(subr_self)->name);
-  fehler(error,GETTEXT("~S (~S ~S): PCRE library lacks pcre_get_stringnumber()"));
+  error(error_condition,GETTEXT("~S (~S ~S): PCRE library lacks pcre_get_stringnumber()"));
 #endif
 }
 
 DEFFLAGSET(pcre_exec_flags, PCRE_ANCHORED PCRE_NOTBOL PCRE_NOTEOL \
            PCRE_NOTEMPTY PCRE_PARTIAL PCRE_DFA_SHORTEST PCRE_DFA_RESTART)
-DEFUN(PCRE:PCRE-EXEC,pattern subject &key :WORK-SPACE :DFA :BOOLEAN :OFFSET \
-      :ANCHORED :NOTBOL :NOTEOL :NOTEMPTY :PARTIAL :DFA-SHORTEST :DFA-RESTART)
+DEFUN(PCRE:PCRE-EXEC,pattern subject &key WORK-SPACE DFA BOOLEAN OFFSET \
+      ANCHORED NOTBOL NOTEOL NOTEMPTY PARTIAL DFA-SHORTEST DFA-RESTART)
 { /* match the SUBJECT string against a pre-compiled PATTERN;
      return a vector of MATCH structures or NIL if no matches */
   int options = pcre_exec_flags();
@@ -351,7 +351,7 @@ DEFUN(PCRE:PCRE-EXEC,pattern subject &key :WORK-SPACE :DFA :BOOLEAN :OFFSET \
   pcre *c_pat;
   pcre_extra *study;
   skipSTACK(3); /* drop all options */
-  check_pattern(STACK_1,&c_pat,&study);
+  check_pattern(&STACK_1,&c_pat,&study);
   begin_system_call();
   ret = pcre_fullinfo(c_pat,study,PCRE_INFO_CAPTURECOUNT,&capture_count);
   end_system_call();
