@@ -15,6 +15,8 @@
    fundamental-binary-output-stream
    ;; general generic functions:
    stream-position
+   stream-read-sequence
+   stream-write-sequence
    ;; Generic functions for character input:
    stream-read-char
    stream-unread-char
@@ -61,7 +63,7 @@
   (let ((clos::*allow-mixing-metaclasses* t))
     (clos:defclass fundamental-stream (stream clos:standard-object)
       (($open :type boolean :initform t) ; whether the stream is open
-       ($reval :type boolean :initform nil) ; whether read-eval is allowed
+       ($fasl :type boolean :initform nil) ; read-eval is allowed; \r=#\Return
        ($penl :type boolean :initform nil) ; whether an elastic newline is pending
 ) ) ) )
 
@@ -156,6 +158,23 @@
   (:method ((stream fundamental-stream) position)
     (clos:no-applicable-method #'stream-position stream position)))
 
+
+(clos:defgeneric stream-read-sequence (sequence stream &key start end)
+  (:method (sequence (stream fundamental-binary-input-stream) &key start end)
+    (stream-read-byte-sequence sequence stream start end))
+  (:method (sequence (stream fundamental-character-input-stream) &key start end)
+    (stream-read-char-sequence sequence stream start end))
+  (:method (sequence (stream fundamental-input-stream) &rest rest)
+    (apply #'sys::%read-sequence sequence stream rest)))
+
+(clos:defgeneric stream-write-sequence (sequence stream &key start end)
+  (:method (sequence (stream fundamental-binary-output-stream) &key start end)
+    (stream-write-byte-sequence sequence stream start end))
+  (:method (sequence (stream fundamental-character-output-stream) &key start end)
+    (stream-write-char-sequence sequence stream start end))
+  (:method (sequence (stream fundamental-output-stream) &rest rest)
+    (apply #'sys::%write-sequence sequence stream rest)))
+
 ;; Generic functions for character input
 
 ; We define the methods on fundamental-input-stream, not
