@@ -1,5 +1,5 @@
 (in-package "EXT")
-(export '(ethe letf letf* with-collect compiled-file-p))
+(export '(ethe letf letf* with-collect compiled-file-p compile-time-value))
 (in-package "SYSTEM")
 ;;; ---------------------------------------------------------------------------
 ;;; Wie THE, nur dass auch im compilierten Code der Typtest durchgeführt wird.
@@ -381,3 +381,19 @@ with compatible bytecodes."
              (and (consp form)
                   (eq (car form) 'SYSTEM::VERSION)
                   (null (eval form))))))))
+
+;;; ---------------------------------------------------------------------------
+;;; http://groups.google.com/group/comp.lang.lisp/browse_thread/thread/7fda163e5e8194f2/65564bd5e2810f01
+(defmacro compile-time-value (expression)
+  "Evaluate the EXPRESSION at compile time, writing the value into the FAS file."
+  (declare (ignore expression))
+  () ; see compiler.lisp:c-COMPILE-TIME-VALUE
+  #+(or) ;; the gensym result leaks into the *.lib and *.fas files
+  (let ((result (gensym "COMPILE-TIME-VALUE-")))
+    `(let ((,result nil))
+       (declare (special ,result))
+       (eval-when (compile)
+         (setq ,result `',(eval ',expression)))
+       (eval-when (compile load eval)
+         (macrolet ((ctv () ,result))
+           (eval-when (load eval) (ctv)))))))
